import { NextFunction, Request, Response } from "express";
import prisma from "../../database/dbclient";
import { z } from "zod";
import bcrypt from "bcryptjs";
import crypto from "crypto";
import { parse } from "dotenv";

export async function getUserData(
  req: Request,
  res: Response,
  next: NextFunction
) {
  const user_id = req.user_id;
  try {
    const userData = await prisma.employees.findUnique({
      where: { user_id },
      include: {
        departments: true,
        user: {
          omit: {
            password_hash: true,
            email: true,
          },
        },
        designations: true,
        leave_approvers: {
          where: { is_active: 1 },
        },
      },
    });

    return res.status(200).json(userData);
  } catch (err) {
    console.log(
      "GET > Get user profile information > Failed to get user profile information >",
      err
    );
    next(err);
  }
}

export async function getAllSubAdmin(
  req: Request,
  res: Response,
  next: NextFunction
) {
  try {
    const userData = await prisma.users.findMany({
      where: {
        user_roles: {
          roles: {
            role_name: {
              // More roles here...
              in: ["Super Admin", "Admin"],
            },
          },
        },
      },
      orderBy: {
        user_id: "desc",
      },
      select: {
        first_name: true,
        last_name: true,
        middle_name: true,
        status: true,
        user_id: true,
        email: true,
        access_permissions: {
          select: {
            permission: true,
            permission_id: true,
          },
        },
        user_roles: {
          include: {
            roles: true,
          },
        },
      },
    });

    return res.status(200).json(userData);
  } catch (err) {
    console.log(
      "GET > Get user profile information > Failed to get user profile information >",
      err
    );
    next(err);
  }
}

export const getAuthData = async (req: Request, res: Response) => {
  const user_id = req.user_id;

  try {
    const user = await prisma.users.findUnique({
      where: { user_id: user_id },
      include: {
        user_roles: {
          select: {
            roles: {
              select: { role_name: true },
            },
          },
        },
        // Company data is necessary to redirect employees and company admins
        usercompany: true,
      },
    });

    res.json({ user: user });
  } catch (error) {
    res.status(500).json({ message: "Error logging in", error });
  }
};
///TODo need to change location
export async function getAllPermissions(
  req: Request,
  res: Response,
  next: NextFunction
) {
  try {
    const permissions = await prisma.permissions.findMany({});

    return res.status(200).json(permissions);
  } catch (err) {
    next(err);
  }
}

//create sub-admin

const createAdmin = z.object({
  email: z.string().email(),
  first_name: z.string(),
  last_name: z.string(),
  middle_name: z.string(),
  status: z.enum(["active", "inactive", "suspended"]).optional().nullable(),
  userAccess: z.array(z.coerce.number().positive()),
});

export const update_Sub_Admin = async (
  req: Request,
  res: Response,
  next: NextFunction
) => {
  try {
    // Validate input data
    const validatedData = createAdmin.parse(req.body);

    validatedData.email;

    // Create the new user
    var newUser = await prisma.users.update({
      where: {
        email: validatedData.email,
      },
      data: {
        email: validatedData.email,
        first_name: validatedData.first_name,
        last_name: validatedData.last_name,
        middle_name: validatedData.middle_name,
        status: validatedData.status ?? "inactive",
      },
    });

    var user_id = newUser.user_id;

    // delete old access
    var userAccess = await prisma.user_access.deleteMany({
      where: {
        user_id: user_id,
      },
    });

    // now the access
    const userRoleData = validatedData.userAccess.map((permission_id) => ({
      user_id,
      permission_id,
    }));

    var userAccess = await prisma.user_access.createMany({
      data: userRoleData,
    });

    return res.status(201).json({ success: true, data: "User updated" });
  } catch (error) {
    next(error);
  }
};

export const create_Sub_Admin = async (
  req: Request,
  res: Response,
  next: NextFunction
) => {
  try {
    // Validate input data
    const validatedData = createAdmin.parse(req.body);

    validatedData.email;

    // Create a new job
    const existingUser = await prisma.users.findUnique({
      where: { email: validatedData.email },
    });

    if (existingUser) {
      return res.status(400).json({ message: "Email already exists" });
    }
    const password = await generateHashedPassword("123456");

    // Create the new user
    var newUser = await prisma.users.create({
      data: {
        email: validatedData.email,
        password_hash: password,
        first_name: validatedData.first_name,
        last_name: validatedData.last_name,
        middle_name: validatedData.middle_name,
        status: validatedData.status ?? "inactive",
      },
    });

    const user_roles = await prisma.user_roles.create({
      data: {
        user_id: newUser.user_id,
        role_id: 2, // 2 means normal admin
      },
    });
    var user_id = newUser.user_id;
    // now the access
    const userRoleData = validatedData.userAccess.map((permission_id) => ({
      user_id,
      permission_id,
    }));

    var userAccess = await prisma.user_access.createMany({
      data: userRoleData,
    });

    return res.status(201).json({ success: true, data: "User Added" });
  } catch (error) {
    next(error);
  }
};

const generateHashedPassword = async (
  plainTextPassword: string
): Promise<string> => {
  console.log(plainTextPassword);
  const saltRounds = 10; // Defines the complexity of the hash
  const hashedPassword = await bcrypt.hash(plainTextPassword, saltRounds);
  return hashedPassword;
};
